import { Express, Router } from 'express';
import pino from 'pino';
import { TypeWithID, Collection, CollectionModel } from './collections/config/types';
import { SanitizedConfig, EmailOptions, InitOptions } from './config/types';
import { TypeWithVersion } from './versions/types';
import { PaginatedDocs } from './mongoose/types';
import { Operations } from './init/bindOperations';
import { RequestHandlers } from './init/bindRequestHandlers';
import { PayloadAuthenticate } from './express/middleware/authenticate';
import { Globals } from './globals/config/types';
import { GraphQLResolvers } from './graphql/bindResolvers';
import { ErrorHandler } from './express/middleware/errorHandler';
import { encrypt, decrypt } from './auth/crypto';
import { BuildEmailResult, Message } from './email/types';
import { Preferences } from './preferences/types';
import { Options as CreateOptions } from './collections/operations/local/create';
import { Options as FindOptions } from './collections/operations/local/find';
import { Options as FindByIDOptions } from './collections/operations/local/findByID';
import { Options as UpdateOptions } from './collections/operations/local/update';
import { Options as DeleteOptions } from './collections/operations/local/delete';
import { Options as FindVersionsOptions } from './collections/operations/local/findVersions';
import { Options as FindVersionByIDOptions } from './collections/operations/local/findVersionByID';
import { Options as RestoreVersionOptions } from './collections/operations/local/restoreVersion';
import { Options as FindGlobalVersionsOptions } from './globals/operations/local/findVersions';
import { Options as FindGlobalVersionByIDOptions } from './globals/operations/local/findVersionByID';
import { Options as RestoreGlobalVersionOptions } from './globals/operations/local/restoreVersion';
import { Result } from './auth/operations/login';
/**
 * @description Payload
 */
export declare class Payload {
    config: SanitizedConfig;
    collections: {
        [slug: string]: Collection;
    };
    versions: {
        [slug: string]: CollectionModel;
    };
    graphQL: {
        resolvers: GraphQLResolvers;
    };
    preferences: Preferences;
    globals: Globals;
    logger: pino.Logger;
    express: Express;
    router: Router;
    emailOptions: EmailOptions;
    email: BuildEmailResult;
    sendEmail: (message: Message) => Promise<unknown>;
    secret: string;
    mongoURL: string;
    local: boolean;
    encrypt: typeof encrypt;
    decrypt: typeof decrypt;
    operations: Operations;
    errorHandler: ErrorHandler;
    authenticate: PayloadAuthenticate;
    requestHandlers: RequestHandlers;
    /**
     * @description Initializes Payload
     * @param options
     */
    init(options: InitOptions): void;
    getAdminURL: () => string;
    getAPIURL: () => string;
    /**
     * @description Performs create operation
     * @param options
     * @returns created document
     */
    create: <T = any>(options: CreateOptions<T>) => Promise<T>;
    /**
     * @description Find documents with criteria
     * @param options
     * @returns documents satisfying query
     */
    find: <T extends TypeWithID = any>(options: FindOptions) => Promise<PaginatedDocs<T>>;
    findGlobal: <T>(options: any) => Promise<T>;
    updateGlobal: <T>(options: any) => Promise<T>;
    /**
     * @description Find global versions with criteria
     * @param options
     * @returns versions satisfying query
     */
    findGlobalVersions: <T extends TypeWithVersion<T> = any>(options: FindGlobalVersionsOptions) => Promise<PaginatedDocs<T>>;
    /**
     * @description Find global version by ID
     * @param options
     * @returns global version with specified ID
     */
    findGlobalVersionByID: <T extends TypeWithVersion<T> = any>(options: FindGlobalVersionByIDOptions) => Promise<T>;
    /**
     * @description Restore global version by ID
     * @param options
     * @returns version with specified ID
     */
    restoreGlobalVersion: <T extends TypeWithVersion<T> = any>(options: RestoreGlobalVersionOptions) => Promise<T>;
    /**
     * @description Find document by ID
     * @param options
     * @returns document with specified ID
     */
    findByID: <T extends TypeWithID = any>(options: FindByIDOptions) => Promise<T>;
    /**
     * @description Update document
     * @param options
     * @returns Updated document
     */
    update: <T = any>(options: UpdateOptions<T>) => Promise<T>;
    delete: <T extends TypeWithID = any>(options: DeleteOptions) => Promise<T>;
    /**
     * @description Find versions with criteria
     * @param options
     * @returns versions satisfying query
     */
    findVersions: <T extends TypeWithVersion<T> = any>(options: FindVersionsOptions) => Promise<PaginatedDocs<T>>;
    /**
     * @description Find version by ID
     * @param options
     * @returns version with specified ID
     */
    findVersionByID: <T extends TypeWithVersion<T> = any>(options: FindVersionByIDOptions) => Promise<T>;
    /**
     * @description Restore version by ID
     * @param options
     * @returns version with specified ID
     */
    restoreVersion: <T extends TypeWithVersion<T> = any>(options: RestoreVersionOptions) => Promise<T>;
    login: <T extends TypeWithID = any>(options: any) => Promise<Result & {
        user: T;
    }>;
    forgotPassword: (options: any) => Promise<any>;
    resetPassword: (options: any) => Promise<any>;
    unlock: (options: any) => Promise<any>;
    verifyEmail: (options: any) => Promise<any>;
}
declare const payload: Payload;
export default payload;
